/*
 * Input event loop functionality.
 *
 * (c) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/input-event-loop.h>

#include <pthread.h>
#include <pthread-l4.h>

#include <l4/re/event>

/* Main loop for event threads. */

void *
Input_event_loop::event_loop(void *data)
{
  Input_event_loop *self = reinterpret_cast<Input_event_loop *>(data);

  while (1) self->handle();

  return 0;
}

/* Handler for incoming interrupts signalling the presence of events. */

void
Input_event_loop::handle()
{
  L4Re::Event_buffer::Event *event;

  if (!l4_error(_irq->receive()))
  {
    while ((event = _event_buffer.next()))
    {
      _handler(*event, _priv);
      event->free();
    }
  }
}

/* Event loop initiation. */

void
Input_event_loop::start()
{
  pthread_attr_t thread_attr;
  struct sched_param sp;

  /* Thread initialisation boilerplate. */

  if (pthread_attr_init(&thread_attr))
    return;

  sp.sched_priority = _priority;
  pthread_attr_setschedparam(&thread_attr, &sp);
  pthread_attr_setschedpolicy(&thread_attr, SCHED_L4);
  pthread_attr_setinheritsched(&thread_attr, PTHREAD_EXPLICIT_SCHED);

  /* Provide this instance as the private data. */

  pthread_create(&_pthread, &thread_attr, event_loop, this);

  _irq->bind_thread(Pthread::L4::cap(_pthread), 0xDEAD);
}
